/*
   kModPlayer - Qt-Based Mod Player
   
   $Id: kmodplayer.cpp,v 1.3 1997/10/25 23:23:32 wuebben Exp $

   Copyright 1997 Bernd Johannes Wuebben math.cornell.edu

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

 */


#include "kmodplayer.h"
#include "playlist.h"

#include "bitmaps/playpaus.xbm"
#include "bitmaps/stop.xbm"
#include "bitmaps/repeat.xbm"
#include "bitmaps/nexttrk.xbm"
#include "bitmaps/prevtrk.xbm"
#include "bitmaps/ff.xbm"
#include "bitmaps/rew.xbm"
#include "bitmaps/poweroff.xbm"
#include "bitmaps/eject.xbm"
#include "bitmaps/modlogo.xbm"
#include "bitmaps/speedup.xbm"
#include "bitmaps/speeddown.xbm"

#include "kldialog.h"
#include "klhelper.h"
#include "kltools.h"
#include "kllabel.h"

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <math.h>

///////////////////////////////////////////////////////////////////////////
// MikMod Interface
///////////////////////////////////////////////////////////////////////////

extern "C" {
#include "mikmod.h"
#include "ui.h"
}

#include "kwm.h"
#include "qtooltip.h"


extern "C"	void	process();

extern "C" 	void 	MD_RegisterPlayer(void (*)(void));
extern "C" 	void 	ML_RegisterLoader(LOADER *ldr);
extern "C" 	void 	MD_RegisterDriver(DRIVER *drv);
extern "C" 	void 	MD_InfoDriver(void);

extern "C" 	void 	ML_InfoLoader(void);
extern "C" 	void 	MD_SetBPM(UBYTE bpm);
extern "C"  BOOL 	MD_Init(void);
extern "C"  void 	MD_Exit(void);
extern "C"  void 	MD_PlayStart(void);
extern "C"  void 	MD_PlayStop(void);
extern "C"  void 	MD_SetBPM(UBYTE bpm);
extern "C"  void 	MD_Update(void);
extern "C" 	int   MP_Ready(void);
extern "C" 	void	ML_Free(UNIMOD *mf);
extern "C" 	void 	MP_Init(UNIMOD *m);
extern "C" 	UNIMOD  *ML_LoadFN(char *filename);
extern "C" 	void 	PlayNote(void);
extern "C" 	void 	PlayEffects(void);
extern "C" 	void 	StartEnvelope(ENVPR *t,UBYTE flg,UBYTE pts,UBYTE sus,
			      UBYTE beg,UBYTE end,ENVPT *p);
extern "C" 	SWORD 	ProcessEnvelope(ENVPR *t,SWORD v,UBYTE keyon);
extern "C" 	SWORD 	DoPan(SWORD envpan,SWORD pan);
extern "C" 	long 	GetFreq2(long period);

extern "C" 	void 	tickhandler(void); 

extern SBYTE globalvolume;
extern float old_bpm;
extern UNIMOD *pf;                     /* <- this modfile is being played */
extern UWORD reppos;                   /* patternloop position */
extern UWORD repcnt;                   /* times to loop */
extern UWORD vbtick;                   /* tick counter */
extern UWORD patbrk;                   /* position where to start a new pattern */
extern UBYTE patdly;                   /* patterndelay counter */
extern UBYTE patdly2;                  /* patterndelay counter */
extern UWORD numrow;                   /* number of rows on current pattern */
extern SWORD posjmp;                   /* flag to indicate a position jump is needed...
														changed since 1.00: now also indicates the
											direction the position has to jump to:

								0: Don't do anything
								1: Jump back 1 position
								2: Restart on current position
								3: Jump forward 1 position
								*/
extern int   forbid;                   /* forbidflag */
extern int isfirst;


extern AUDTMP mp_audio[32];            /* max 32 channels */
extern UBYTE  mp_bpm;                  /* beats-per-minute speed */
extern UWORD  mp_patpos;               /* current row number (0-255) */
extern SWORD  mp_sngpos;               /* current song position */
extern UWORD  mp_sngspd;               /* current songspeed */
extern UWORD  mp_channel;              /* channel it's working on */
extern BOOL   mp_extspd;               /* extended speed flag, default enabled */
extern BOOL   mp_panning;              /* panning flag, default enabled */
extern BOOL   mp_loop;                 /* loop module ? */
extern UBYTE  mp_volume;               /* song volume (0-100) (or user volume) */

extern AUDTMP *a;                      /* current AUDTMP it's working on */

int 	myargc;
char 	**myargv;
int 	myoptind;
bool 	play_commandline_mods 	= false;

KModPlayer* 	thisapp;
KMod*	kmodplayer;
int    	mysec;
int 	mymin;
char 	*tottime;
int 	count_pattern, count_song;
int 	gui_result;
int  	gui_result_storage ;

// DDX/DDV data
QString current_playlist;
int 	playlist_x;
int 	playlist_y;

#define NUM_CUSTOM_COLORS 18

QColor custom_colors[NUM_CUSTOM_COLORS];

char helptext[]=

"Available switches (CaSe SeNsItIvE!):\n"
"\n"
"  -d x    use device-driver #x for output (0 is autodetect). Default=0\n"
"  -ld     List all available device-drivers\n"
"  -ll     List all available loaders\n"
"  -x      disables protracker extended speed\n"
"  -p      disables panning effects (9fingers.mod)\n"
"  -v xx   Sets volume from 0 (silence) to 100. Default=100\n"
"  -f xxxx Sets mixing frequency. Default=44100\n"
"  -m      Force mono output (so sb-pro can mix at 44100)\n"
"  -8      Force 8 bit output\n"
"  -i      Use interpolated mixing\n"
"  -r      Restart a module when it's done playing";

#ifdef ULTRA
extern "C" void processPosChange(struct echoData *echo) ;
extern "C" void processFreqChange(struct echoData *echo); 
extern "C" int use_card;

void processPosChange(struct echoData *echo) 
{
  /*	extern char output_buf_4[256];
	sprintf(output_buf_4, "\rPattern/Pos: %03i/%03i ", echo->data1, echo->data2);
	printw("%s",output_buf_4);
	refresh();
	*/
}

void processFreqChange(struct echoData *echo) 
{
}

#endif /*ULTRA*/

///////////////////////////////////////////////////////////////////////////
// main()
///////////////////////////////////////////////////////////////////////////

int main(int argc,char *argv[])
{
  KModPlayer kmodplayer( argc,argv);

  return thisapp->exec();
}

///////////////////////////////////////////////////////////////////////////
// KModPlayer Application
///////////////////////////////////////////////////////////////////////////
void KModPlayer::makedirs()
{

  QDir dir;
  QString d;

  d = getHomeDir();

  d += "/.kde";
  dir.setPath(d.data());
  if(!dir.exists()){
    dir.mkdir(d.data());
    chown(d.data(),getuid(),getgid());
    chmod(d.data(),S_IRUSR | S_IWUSR | S_IXUSR);
  }

  d += "/share";
  dir.setPath(d.data());
  if(!dir.exists()){
    dir.mkdir(d.data());
    chown(d.data(),getuid(),getgid());
    chmod(d.data(),S_IRUSR | S_IWUSR | S_IXUSR);
  }

  d += "/apps";
  dir.setPath(d.data());
  if(!dir.exists()){
    dir.mkdir(d.data());
    chown(d.data(),getuid(),getgid());
    chmod(d.data(),S_IRUSR | S_IWUSR | S_IXUSR);
  }

  d += "/kmodplayer" ;

  dir.setPath(d.data());
  if(!dir.exists()){
    dir.mkdir(d.data());
    chown(d.data(),getuid(),getgid());
    chmod(d.data(),S_IRUSR | S_IWUSR | S_IXUSR);
  }
  
}

void KModPlayer::forcePlaying(int songnr) 
{
  song_number = songnr;
  if( getState() == KModPlayer::Paused ) 
    setState(KModPlayer::Playing);
  if( getState() == KModPlayer::Playing )
    change_song = TRUE;
  else
    mainWindow->playClicked();
}

QString KModPlayer::getHomeDir() {

  struct passwd *pwd;
  pwd = getpwuid(getuid());

  if(pwd == NULL)
    return QString("/");
  
  QString s = pwd->pw_dir;
  if(s.right(1) != "/")
    s += "/";
  return s;

}

void KModPlayer::parseOptions(int nargc,char *nargv[])
{
  /* Parse option switches using standard getopt function: */
  int cmderr=0;                   /* error in commandline flag */
  int morehelp=0;                 /* set if user wants more help */
  int t;

  while( !cmderr &&
	 (t=getopt(nargc,nargv,"ohxpm8irv:f:l:d:")) != EOF ){

    switch(t){
      
    case 'd':
      md_device=atoi(optarg);
      break;
      
    case 'l':
      if(optarg[0]=='d') MD_InfoDriver();
      else if(optarg[0]=='l') ML_InfoLoader();
      else{
				cmderr=1;
				break;
      }
      exit(0);
      
    case 'r':
      mp_loop=1;
      break;
      
    case 'm':
      md_mode&=~DMODE_STEREO;
      break;
      
    case '8':
      md_mode&=~DMODE_16BITS;
      break;
      
    case 'i':
      md_mode|=DMODE_INTERP;
      break;
      
    case 'x':
      mp_extspd=0;
      break;
      
    case 'p':
      mp_panning=0;
      break;
      
    case 'v':
      if((mp_volume=atoi(optarg))>100) mp_volume=100;
      break;
      
    case 'f':
      md_mixfreq=atol(optarg);
      break;
      
    case 'h':
      morehelp=1;
      cmderr=1;
      break;
      
    case '?':
      puts("\07Invalid switch or option needs an argument\n");
      cmderr=1;
      break;
    }
  }

  if(cmderr ){

    printf("Usage: %s [switches] <mymod.mod> ... \n",nargv[0]);
    
    if(morehelp)
      puts(helptext);
    else
      printf("Type %s -h for more help.\n",nargv[0]);

    exit(-1);
  }
}


KModPlayer::KModPlayer(int argc, char **argv) : KApplication(argc,argv,"kmodplayer")
{
  thisapp = this;
	parseOptions(argc,argv);
  makedirs();

  change_song = FALSE;	// When set to TRUE, exchange the current song with the next/prev

  blink = 0;

  mp_loop = 0;
  doquit = 0;
  go_next_mod = FALSE;
  song_number = 0;

	setState( Ready );

  setFont( QFont( "helvetica", 12 ), TRUE );
  
  mainWindow = new KMod; 
  enableSessionManagement(true);
  setWmCommand(argv[0]);      
  setTopWidget(mainWindow);
  setMainWidget(mainWindow);

	initModplayer();

  /*  initialize volume and tempo multipliers */
  
  speed_constant=1.0;

  if(optind < argc){
        
    play_commandline_mods = true;
    myoptind = optind;
    myargc = argc;
    myargv = argv;
  }

  deviceAlloced = FALSE;

	mainWindow->show();
  mainWindow->updateUI();
  processEvents();
  PlayCommandlineMods();
  dockWindow = new DockWidget("dockw");
  dockWindow->dock();
}


void KModPlayer::initModplayer()
{
  md_mixfreq      = 44100;                     /* standard mixing freq */
  md_dmabufsize   = 32768;                     /* standard dma buf size */
  md_mode         = DMODE_16BITS|DMODE_STEREO; /* standard mixing mode */
  md_device       = 0; 

  ML_RegisterLoader((LOADER*)&load_m15);/* if you use m15load,register it s first! */
  ML_RegisterLoader((LOADER*)&load_mod);
  ML_RegisterLoader((LOADER*)&load_mtm);
  ML_RegisterLoader((LOADER*)&load_s3m);
  ML_RegisterLoader((LOADER*)&load_stm);
  ML_RegisterLoader((LOADER*)&load_ult);
  ML_RegisterLoader((LOADER*)&load_uni);
  ML_RegisterLoader((LOADER*)&load_xm);
  
  MD_RegisterDriver((DRIVER*)&drv_nos);
  MD_RegisterDriver((DRIVER*)&drv_raw);
  
#ifdef SUN
  MD_RegisterDriver((DRIVER*)&drv_sun);
#elif defined(SOLARIS)
  MD_RegisterDriver((DRIVER*)&drv_sun);
#elif defined(__alpha)
  MD_RegisterDriver((DRIVER*)&drv_AF);
#elif defined(LINUX)
  MD_RegisterDriver((DRIVER*)&drv_vox);
#ifdef ULTRA
  MD_RegisterDriver((DRIVER*)&drv_ultra);
#endif /* ULTRA */
#elif defined(__hpux)
  MD_RegisterDriver((DRIVER*)&drv_hp);
#elif defined(AIX)
  MD_RegisterDriver((DRIVER*)&drv_aix);
#elif defined(SGI)
  MD_RegisterDriver((DRIVER*)&drv_sgi);
#endif
  
  MD_RegisterPlayer(tickhandler);

}


bool KModPlayer::allocDevice()
{
  if( deviceAlloced ) 
    return TRUE;

	ASSERT(getState() == Device_Error || getState() == Ready || getState() == Stopped);
	if( !MD_Init() ) 
		setState(Device_Error);
	else
		setState(Ready);
  mainWindow->updateUI();
  deviceAlloced = getState() == Ready;
  return deviceAlloced;
}

void KModPlayer::freeDevice()
{
  if( deviceAlloced ) 
	  MD_Exit();
  deviceAlloced = FALSE;
}


KModPlayer::~KModPlayer()
{
}


bool KModPlayer::startPlaying()
{
  // This routine is called, wenn starting to play a mod
  // It makes sure, that the device is avaible.

  bool errdev = getState() == Device_Error; // Had we an error when allocating device ?

  // Try to get the device
  do {
    allocDevice();
    if(getState() == Device_Error) {
      // we couldn't initialize the driver 
      // this happens e.g. when I still have NAS running
      // let's blink a bit.

      blink = blink ? FALSE : TRUE;
      
      mainWindow->updateUI();
      processEvents();
      sleep(1);
    }
  } while( getState() == Device_Error );

  // Got the driver !!!
  blink = FALSE;
  if(errdev && getState() == Ready)      // Had we an error with an not allocated device ?
    mainWindow->PlayMOD();    // Start playing the mod (if any avail)

  return TRUE;
}


void KModPlayer::stopPlaying()
{
  // This routine is called, when the playing of a MOD has stopped
  // It frees the device, so that other applications can access it
  freeDevice();
}


void KModPlayer::PlayCommandlineMods()
{
  // The routine checks, if there where filenames of MOD files given
  // on the commandline. If so, they will be added to the songlist
  // and the playing will start

  if (play_commandline_mods){
    mainWindow->playlist->clear();

    for(doquit=0; !doquit && myoptind<myargc; myoptind++)
      mainWindow->playlist->append(myargv[myoptind]);
    
    mainWindow->playClicked();
  }
}


void KModPlayer::quit()
{
	KTMainWindow::deleteAll(); // Clean up everything
	KApplication::quit(); 	// Call superclass
}


KModPlayer::ModPlayerState KModPlayer::getState()
{
	return state;
}


void KModPlayer::setState(KModPlayer::ModPlayerState newstate)
{
	state = newstate;
}



static void tickhandler(void)
{

	KMod::MP_HandleTick();    /* play 1 tick of the module */
	MD_SetBPM(mp_bpm);
}


KMod::KMod( QWidget *, const char *name ) :
	KTMainWindow( name )
{
  
  drawPanel();
  loadBitmaps();
  setColors();

  playlist = new Songlist();

  PlaylistDialog::createDialog(playlist);
  KDNDDropZone *dropZone = new KDNDDropZone( this, DndURL );
  connect( dropZone, SIGNAL( dropAction( KDNDDropZone* )), PlaylistDialog::playlistdialog, 
	   SLOT( onDrop( KDNDDropZone*)));

  connect( playPB, SIGNAL(clicked()), SLOT(playClicked()) );
  connect( stopPB, SIGNAL(clicked()), SLOT(stopClicked()) );
  connect( prevPB, SIGNAL(clicked()), SLOT(prevClicked()) );
  connect( nextPB, SIGNAL(clicked()), SLOT(nextClicked()) );
  connect( fwdPB, SIGNAL(clicked()), SLOT(fwdClicked()) );
  connect( bwdPB, SIGNAL(clicked()), SLOT(bwdClicked()) );
  connect( quitPB, SIGNAL(clicked()), SLOT(quitClicked()) );	
  connect( replayPB, SIGNAL(clicked()), SLOT(replayClicked()) );
  connect( ejectPB, SIGNAL(clicked()), SLOT(ejectClicked()) );
  connect( volSB, SIGNAL(valueChanged(int)), SLOT(volChanged(int)));
  connect( aboutPB, SIGNAL(clicked()), SLOT(aboutClicked()));
  connect( slowerPB, SIGNAL(clicked()), SLOT(slowdownslot()));
  connect( fasterPB, SIGNAL(clicked()), SLOT(speedupslot()));

  connect(thisapp,SIGNAL(kdisplayPaletteChanged()),this,SLOT(setColors()));

  adjustSize();
  
  setFixedSize(this->width(),this->height());
  
  count_pattern = 0; 
  count_song = 0;
  gui_result = UI_NONE;
  thisapp->loopmod = false;

  loadPlaylist();
}


KMod::~KMod()
{
}


void KMod::updateUI()
{
  QString helpstr;
  if (thisapp->vol == 100){
    helpstr.sprintf("Vol: %03d%%",thisapp->vol);
  }
  else{
    helpstr.sprintf("Vol: %02d%%",thisapp->vol);
  }
	updateLabel(volLA, savevoltxt, helpstr);

	playtime();

	switch( thisapp->getState() ) 
	{
		case KModPlayer::Device_Error : helpstr = "Driver Error"; break;
	  case KModPlayer::Ready 				: helpstr = "Ready"; break;
		case KModPlayer::Stopped 			: helpstr = "Stopped"; break;
		case KModPlayer::Paused 			: helpstr = "Paused"; break;
		case KModPlayer::Playing			: helpstr = "Playing"; break;
		default					              : ASSERT(FALSE); helpstr = "<< Internal Error >>"; break;
	}
	if( thisapp->blink ) 
		helpstr = "					";
  updateLabel(statusLA, savestatus,helpstr);

  helpstr = "";
  if(thisapp->loopmod)
		 helpstr = "Loop";
  if( helpstr.isEmpty() )
    looplabel->hide();
  else 
    looplabel->show();
 	updateLabel(looplabel,savelooplabel,helpstr);

  helpstr.sprintf("%d bit %s\n%s %u Hz",
		     (md_mode&DMODE_16BITS) ? 16:8,
		     (md_mode&DMODE_INTERP) ? "Interpolated":"Normal",
		     (md_mode&DMODE_STEREO) ? "Stereo":"Mono",
		     md_mixfreq);

  updateLabel(propertiesLA,saveprop,helpstr);
  updateLabel(modlabel,savecurrmodstring,currmodstring);  

  if( playlist ) {
    helpstr.sprintf("Song: %04d/%04d",playlist->count() ? thisapp->song_number+1 : 0,playlist->count());
    updateLabel(song_count_label,savesongcount,helpstr);
  }

  helpstr.sprintf("Spd: %3.0f%%",speed_constant*100);
	updateLabel( speedLA, savespeedtext, helpstr );
}


void KMod::playtime()
{
  int position;
                        
  if(pf && ((thisapp->getState() == KModPlayer::Playing) || (thisapp->getState() == KModPlayer::Paused)))
    position = pf->numpos;
  else 
    position = 0;

	char 	tmptime[100];
  sprintf( tmptime, "%02d:%02d:%03d", position, mp_sngpos , mp_patpos );

 	if( savetime != tmptime ) {
		setLEDs(tmptime);
		savetime = tmptime;
	}
}


void KMod::updateLabel(QLabel *label,QString &oldtext,QString &newtext)
{
	if( oldtext != newtext ) {
		label->setText(newtext);
		oldtext = newtext;
	}
}


void KMod::volChanged( int vol )
{       	
  mp_volume = vol*255/100;
  thisapp->vol = vol;
	updateUI();
}



QPushButton *KMod::makeButton( int x, int y, int w, int h, const char *n ) 
{
	QPushButton *pb = new QPushButton( n, this );
	pb->setGeometry( x, y, w, h );
	return pb;
}
	
void KMod::drawPanel() 
{
	int ix = 0;
	int iy = 0;
	int WIDTH = 90;
	int HEIGHT = 27;
	int SBARWIDTH = 280; //140

	setCaption( "kmodplayer" );
	aboutPB = makeButton( ix, iy, WIDTH, 2 * HEIGHT, "About" );

	ix = 0;
	iy += 2 * HEIGHT;
	ejectPB = makeButton( ix, iy, WIDTH, HEIGHT, "Eject" );

	iy += HEIGHT;
	quitPB = makeButton( ix, iy, WIDTH, HEIGHT, "Quit" );

	ix += WIDTH;
	iy = 0;

	//	backdrop = new MyBackDrop(this,"mybackdrop");
	backdrop = new QWidget(this,"mybackdrop");
	backdrop->setGeometry(ix,iy,SBARWIDTH, 3* HEIGHT);


	ix = WIDTH;
	int D = 10;
	for (int u = 0; u<=8;u++){
	  trackTimeLED[u] = new BW_LED_Number(this );	 
	trackTimeLED[u]->setGeometry( ix  + u*18, iy + D, 23 ,  30 );
	//	  trackTimeLED[u]->setGeometry( ix +3 + u*20, iy+12, 23 ,  6*HEIGHT/5 );
	}
	

	ix = WIDTH;
	iy += 2 * HEIGHT;
	statusLA = new QLabel( this );
	statusLA->setGeometry( WIDTH +2*SBARWIDTH/3 -6, 7, 80, 12 );
	statusLA->setFont( QFont( "Helvetica", 10, QFont::Bold ) );
        statusLA->setAlignment( AlignLeft );
        statusLA->setText("   ");

	
	looplabel = new QLabel( this );
	looplabel->setGeometry( WIDTH +2*SBARWIDTH/3 +39, 7, 60, 12 );
	looplabel->setFont( QFont( "Helvetica", 10, QFont::Bold ) );
        looplabel->setAlignment( AlignLeft );
        looplabel->setText("");

	propertiesLA = new QLabel( this );
	propertiesLA->setGeometry( WIDTH + 2*SBARWIDTH/3 -6,  35, 100, 40 );
	propertiesLA->setFont( QFont( "Helvetica", 10, QFont::Bold ) );
        propertiesLA->setAlignment( AlignLeft );
        propertiesLA->setText("");


	ix += SBARWIDTH/2;

	fasterPB = makeButton( WIDTH + 2*SBARWIDTH/3 , 
			       iy+ HEIGHT, SBARWIDTH/6 , HEIGHT, "F" );

	slowerPB = makeButton( WIDTH + 5*SBARWIDTH/6 , 
			       iy+HEIGHT, SBARWIDTH/6 , HEIGHT, "S" );

	lowerBar = new QPushButton(this);
	lowerBar->setGeometry( WIDTH, 7*HEIGHT/2, 2*SBARWIDTH/3 -1, HEIGHT/2  );

	ix = WIDTH ;
	iy += HEIGHT;

	volLA = new QLabel( this );
        volLA->setAlignment( AlignLeft );
        volLA->setGeometry( WIDTH + 2*SBARWIDTH/3 -6 , 21, 60, 12 );

        volLA->setFont( QFont( "helvetica", 10, QFont::Bold) );
	init_volume = mp_volume*100/255;
  thisapp->vol = init_volume;


	speedLA = new QLabel( this );
  speedLA->setAlignment( AlignLeft );
  speedLA->setGeometry( WIDTH + 2*SBARWIDTH/3 +43, 21, 80, 12 );
  speedLA->setFont( QFont( "helvetica", 10, QFont::Bold) );


	modlabel = new QLabel( this );
  modlabel->setAlignment( AlignLeft );
  modlabel->setGeometry( WIDTH + 10, HEIGHT + 22, 170, 40 );
  modlabel->setFont( QFont( "helvetica", 10, QFont::Bold) );
	modlabel->setText( "" );


	song_count_label = new QLabel( this );
  song_count_label->setAlignment( AlignLeft );
	song_count_label->setGeometry( WIDTH +2*SBARWIDTH/3 -6, 62, 80, 12 );
  song_count_label->setFont( QFont( "helvetica", 10, QFont::Bold) );
	song_count_label->setText( "" );


	iy += HEIGHT / 2 ;
	
	volSB = new QSlider( 0, 100, 5,  init_volume, QSlider::Horizontal, 
        this, "Slider" );
        volSB->setGeometry( WIDTH, 3*HEIGHT, 2*SBARWIDTH/3, HEIGHT /2 );

	iy = 0;
	ix = WIDTH + SBARWIDTH;

	int MWIDTH = 50;

	playPB = makeButton( ix, iy, 2*MWIDTH, HEIGHT, "Play/Pause" );

	iy += HEIGHT;
	stopPB = makeButton( ix, iy, MWIDTH , HEIGHT, "Stop" );

	ix += MWIDTH ;
	replayPB = makeButton( ix, iy, MWIDTH , HEIGHT, "Replay" );

	ix = WIDTH + SBARWIDTH;
	iy += HEIGHT;
	bwdPB = makeButton( ix, iy, MWIDTH , HEIGHT, "Bwd" );

	ix += MWIDTH ;
	fwdPB = makeButton( ix, iy, MWIDTH , HEIGHT, "Fwd" );

	ix = WIDTH + SBARWIDTH;
	iy += HEIGHT;
	prevPB = makeButton( ix, iy, MWIDTH , HEIGHT, "Prev" );

	ix += MWIDTH ;
	nextPB = makeButton( ix, iy, MWIDTH , HEIGHT, "Next" );
}


void KMod::loadBitmaps() {

	QBitmap playBmp( playpause_width, playpause_height, playpause_bits,
			 TRUE );
	QBitmap stopBmp( stop_width, stop_height, stop_bits, TRUE );
	QBitmap prevBmp( prevtrk_width, prevtrk_height, prevtrk_bits, TRUE );
	QBitmap nextBmp( nexttrk_width, nexttrk_height, nexttrk_bits, TRUE );
	QBitmap replayBmp( repeat_width, repeat_height, repeat_bits, TRUE );
	QBitmap fwdBmp( ff_width, ff_height, ff_bits, TRUE );
	QBitmap bwdBmp( rew_width, rew_height, rew_bits, TRUE );
	QBitmap ejectBmp( eject_width, eject_height, eject_bits, TRUE );
	QBitmap quitBmp( poweroff_width, poweroff_height, poweroff_bits, 
			 TRUE );
	QBitmap aboutBmp( modlogo_width, modlogo_height, modlogo_bits, TRUE );
	QBitmap speedupBmp( speedup_width, speedup_height, speedup_bits, TRUE );
	QBitmap speedownBmp( speeddown_width, speeddown_height, speeddown_bits, TRUE );

	playPB->setPixmap( playBmp );
	fasterPB->setPixmap( speedupBmp );
	slowerPB->setPixmap( speedownBmp );
	stopPB->setPixmap( stopBmp );
	prevPB->setPixmap( prevBmp );
	nextPB->setPixmap( nextBmp );
	replayPB->setPixmap( replayBmp );
	fwdPB->setPixmap( fwdBmp );
	bwdPB->setPixmap( bwdBmp );
	ejectPB->setPixmap( ejectBmp );
	quitPB->setPixmap( quitBmp );
	aboutPB->setPixmap( aboutBmp );
}


void KMod::setLEDs(QString symbols){

   for(int i=0;i<=8;i++){
    trackTimeLED[i]->display(symbols.data()[i]);
  }

}


void KMod::cleanUp() 
{
  	exit(0); // Beleave on UNIX -- ResourceTracking ...
}


void KMod::playClicked()
{

  if(!thisapp->startPlaying()) // Is the device availble ? If not, 
    return;                    // wait until it is avaible and then call playClicked() again

  thisapp->doquit = false;

  if( (thisapp->getState() == KModPlayer::Playing)){
		thisapp->setState(KModPlayer::Paused);
		updateUI();
   	return;
  }

  gui_result_storage = 0;

  if((thisapp->getState() == KModPlayer::Paused)){
   thisapp->setState(KModPlayer::Playing);
	 updateUI();
   return;
  }

  if ( !playlist->isEmpty() ) {  
		thisapp->setState(KModPlayer::Playing);
		updateUI();

    thisapp->processEvents();

    do{ 

      do{
        thisapp->setState(KModPlayer::Playing);

        if (thisapp->song_number >= (int)playlist->count())
          thisapp->song_number = 0;

        gui_result = UI_NONE;
        thisapp->processEvents(); 

        if( !playthemod(playlist->at(thisapp->song_number)) ) {
          // An error has accord when playing 
          // Stop playing future mods !
          gui_result_storage = UI_QUIT;
        }

        if( thisapp->getState() == KModPlayer::Stopped )
          break;
        
        if (thisapp->go_next_mod) {
          if (thisapp->song_number + 1 >= (int)playlist->count())
            break;
          nextClicked();
        }

        PlaylistDialog::playlistdialog->selectSong(thisapp->song_number);

      } while(gui_result_storage != UI_QUIT);
    
    } while(thisapp->loopmod && (gui_result_storage != UI_QUIT) );

    if( thisapp->getState() != KModPlayer::Stopped )
		  thisapp->setState(KModPlayer::Ready);

    // when the mod has been played to the end and wasn't stopped by the user
    // display READY

    resetPos();
  }
  else{
    KMsgBox::message(this, "KModPlayer - Sorry", 
                    "You must select a mod first.", KMsgBox::INFORMATION );
  } 

  thisapp->stopPlaying();
}


void KMod::stopClicked()
{
  if((thisapp->getState() == KModPlayer::Playing) || (thisapp->getState() == KModPlayer::Paused)){
    mp_sngpos = 0; // these are here to make the LED display well behaved.
    mp_patpos = 0;
  }

  thisapp->setState(KModPlayer::Stopped);
  thisapp->loopmod = false;
  gui_result = UI_QUIT;
  gui_result_storage = UI_QUIT;
  count_song = 0;
  resetPos();
}

void KMod::resetPos()
{
  mp_sngpos = 0;
	updateUI();
}

void KMod::prevClicked(){

  gui_result = UI_PREVIOUS_SONG;
  thisapp->song_number --;

  if (thisapp->song_number < 0)
    thisapp->song_number =  playlist->count();

  if( thisapp->getState() == KModPlayer::Paused )
    thisapp->setState( KModPlayer::Playing );

  updateSongName();
}


void KMod::slowdownslot(){

  if(thisapp->getState() != KModPlayer::Playing)
    return;
  gui_result = UI_SLOW_DOWN;
}

void KMod::speedupslot()
{
  if(thisapp->getState() != KModPlayer::Playing)
    return;
  gui_result = UI_SPEED_UP;
}
 
void KMod::nextClicked()
{
  gui_result = UI_NEXT_SONG;
  if( PlaylistDialog::playlistdialog->isRandomPlay() )
    thisapp->song_number = (int)(((double) playlist->count())*rand()/(RAND_MAX+1.0));
  else
    thisapp->song_number ++;

  if (thisapp->song_number >= (int)playlist->count())
    thisapp->song_number = 0;

  if( thisapp->getState() == KModPlayer::Paused )
    thisapp->setState( KModPlayer::Playing );

  updateSongName();
}

void KMod::fwdClicked()
{
  gui_result = UI_JUMP_TO_NEXT_PATTERN;
}

void KMod::bwdClicked()
{
  gui_result = UI_JUMP_TO_PREV_PATTERN;
}

void KMod::quitClicked() 
{
  thisapp->freeDevice();
  cleanUp();
}

void KMod::replayClicked()
{
  thisapp->loopmod = !thisapp->loopmod;	
}


void KMod::loadPlaylist()
{
  current_playlist = "default";
  
  QString home = QDir::homeDirPath();
  home = home + "/.kde/share/apps/kmodplayer";
  
  QDir savedir(home.data());

  QString defaultlist;
  defaultlist = home + "/" + current_playlist;
  
  playlist->loadSonglist(defaultlist);
  PlaylistDialog::playlistdialog->applyList();
  updateSongName();
}


void KMod::ejectClicked()
{
  PlaylistDialog::showPlaylist();
}


void KMod::PlayMOD()
{
  if (!playlist->isEmpty() && (thisapp->getState() != KModPlayer::Playing))
    playClicked(); 
}

void KMod::aboutClicked()
{
  QString pixname = thisapp->kde_datadir() +  QString("/kmodplayer/pics/kmodlogo.xpm"); 
  KLDialog *dialog = new KLDialog();
  dialog->setCaption( "About KModPlayer" );
  KLLabel *label, *label2, *labeltitle;
  KLButton *ok;
  KLChild &child = (klVGroup()
                    << ::setSpaceBorder
                    << (klGroupBox("About KModPlayer",false)
                       << (labeltitle = klLabel("KModPlayer " KMODVERSION))
                       << klLabel("Written by\t")
                       << (klHGroup()
                         << (klVGroup()
                            << klHVSpace()
                            << new KLLabel("",pixname,KLLabel::Image)
                            << klHVSpace())
                         << (label = klLabel("")))
                       << (*(label2 = klLabel("")) << setWeight << 1))
                     << (klHGroup()
                        << ::setSameSize 
                        << klHVSpace()
                        << klHVSpace()
                        << klHVSpace()
                        << (ok = klButton("Ok"))));
  dialog->setRootChild(&child);
  connect(ok,SIGNAL(clicked()),dialog,SLOT(reject()));
  labeltitle->setFont(QFont("Helvetica",24,QFont::Bold,true));
  label->setText("Bernd Johannes Wuebben\n"
		             "wuebben@math.cornell.edu\n"
                  "Emmeran \"Emmy\" Seehuber\n" 
                  "es@mylius.com\n\n"
		             "(C) 1997/1998");
  label2->setText("KModPlayer is based on MikMod 2.13\n"
                 "(C) Jean Paul Mikkers\nLogo was done by Harald Lapp");
  dialog->show();
}


void KMod::setColors()
{
	QColor background_color = black;
	QColor led_color = QColor(226,224,255);

  backdrop->setBackgroundColor(background_color);
  
  QColorGroup colgrp( led_color, background_color, yellow,yellow , yellow,
                        led_color, white );
  
  statusLA->setPalette( QPalette(colgrp,colgrp,colgrp) );
  looplabel->setPalette( QPalette(colgrp,colgrp,colgrp) );
  propertiesLA->setPalette( QPalette(colgrp,colgrp,colgrp) );
  volLA->setPalette( QPalette(colgrp,colgrp,colgrp) );
  speedLA->setPalette( QPalette(colgrp,colgrp,colgrp) );
  modlabel->setPalette( QPalette(colgrp,colgrp,colgrp) );
  song_count_label->setPalette( QPalette(colgrp,colgrp,colgrp) );


  for (int u = 0; u<=8;u++){
    trackTimeLED[u]->setLEDoffColor(background_color);
    trackTimeLED[u]->setLEDColor(led_color,background_color);
  }
}

void KMod::updateSongName()
{ 
  if( currmodstring == currmodname )
    currmodstring = "";
  if( playlist && (playlist->count() > 0) )  {
    QFileInfo fi(playlist->at(thisapp->song_number));
    currmodname = "";
    currmodname.sprintf("\n%s", (const char *)fi.fileName());
  }
  else
    currmodname = "";

  if( currmodstring.isEmpty() ) 
    currmodstring = currmodname;
  updateUI();
}


bool KMod::playthemod(QString modname)
{
  updateSongName();
  UNIMOD *mf;
    
  {
    QFileInfo fi(playlist->at(thisapp->song_number));
    currmodstring = "";
    currmodstring.sprintf("Loading/Decrunching ...\n%s", (const char *)fi.fileName() );
    updateUI();
    thisapp->processEvents(); // Let the GUI be updated
    currmodstring = "";
  }

  mf = ML_LoadFN(modname.data());

  if( mf == NULL ){
    QString message;
    message.sprintf("kMod error:\n%s.",myerr);	  
    KMsgBox::message(this, "KModPlayer - Sorry", 
                    message.data(), KMsgBox::EXCLAMATION);
    updateSongName();
    return FALSE;
  }


  /* initialize modplayer to play this module */

  MP_Init( mf );


  {
    QFileInfo fi(playlist->at(thisapp->song_number));
    QString modstring;
    QString modaux;
    modaux.sprintf("%s",mf->songname ? mf->songname : (const char *)fi.fileName());
    modaux = modaux.stripWhiteSpace(); 
    currmodstring = "";
    currmodstring.sprintf("%s\n%s", mf->modtype ? mf->modtype : "Unknown",  modaux.data() );
  }
    
  updateUI();

  /*	set the number of voices to use.. you
	could add extra channels here (e.g. md_numchn=mf->numchn+4; )
	to use for your own soundeffects:
  */
  
  md_numchn=mf->numchn;
  
  MD_PlayStart();

  thisapp->go_next_mod = TRUE;

  while(!MP_Ready() && !thisapp->doquit){ 

    if (thisapp->getState() == KModPlayer::Playing){
      MD_Update();
    }
    else {
      if( thisapp->getState() == KModPlayer::Stopped ) // When stoping playing -- stop this loop,
        break;                                         // so that the device can be freed
      usleep(500); 	// Dont block the processor while pausing !
    }
    
    if(thisapp->change_song){ 		          // TRUE when the next or previous
      thisapp->change_song = FALSE;	        // mod is selected
      thisapp->go_next_mod = FALSE;
      break;
    }
    
    updateUI(); // Just update the state display

    thisapp->processEvents(); // And let accept UI input !
  }

  MD_PlayStop();          /* stop playing */
  ML_Free(mf);            /* and free the module */

  currmodstring = "";
  pf = NULL;
  updateSongName();
  return TRUE;
}


void KMod::closeEvent( QCloseEvent *e )
{
  e->ignore();                            
  quitClicked();
}



void KMod::MP_HandleTick(void)
{
  ULONG tmpvol;
  int t,tr;
    
  //	pause_flag=-128;
  
  if(isfirst){           
  
    /* don't handle the very first ticks, this allows the
       other hardware to settle down so we don't loose any 
       starting notes
       */
    
    isfirst--;
    return;
  
  }
    
  if(forbid) return;      /* don't go any further when forbid is true */
    
  if(MP_Ready()) return;
    
  if(++vbtick>=mp_sngspd){
    
    mp_patpos++;
    vbtick=0;
    
    if(patdly){
      patdly2=patdly;
      patdly=0;
    }
    
    if(patdly2){
      
      /* patterndelay active */
      
      if(--patdly2) {
	      mp_patpos--;    /* so turn back mp_patpos by 1 */
      }
    }
      
    /* Do we have to get a new patternpointer ?
       (when mp_patpos reaches 64 or when
       a patternbreak is active). Also check for 255 - if mod 
       is broken it will continue forever otherwise */
    
    if( mp_patpos == numrow || mp_patpos > 255 ) 
      posjmp=3;
      
    if( posjmp ) {
      mp_patpos=patbrk;
      mp_sngpos+=(posjmp-2);
      patbrk=posjmp=0;
      if(mp_sngpos>=pf->numpos){
        if(TRUE) return;    // what is going on here? -- Bernd
        if(!mp_loop) return;
        mp_sngpos=pf->reppos;
      }
      if(mp_sngpos<0) mp_sngpos=pf->numpos-1;
    }
    
    if(!patdly2){
      for(t=0;t<pf->numchn;t++){
        tr=pf->patterns[(pf->positions[mp_sngpos]*pf->numchn)+t];
        numrow=pf->pattrows[pf->positions[mp_sngpos]];
        
        mp_channel=t;
        a=&mp_audio[t];
        a->row=(tr<pf->numtrk)  
          ? UniFindRow(pf->tracks[tr],mp_patpos) 
          : (UBYTE*) NULL;
        
        PlayNote();
      }
      
      count_pattern++;
      count_song++;
      
      switch(gui_result)
      {
        case UI_NEXT_SONG:
          thisapp->change_song = TRUE;
          gui_result = UI_NONE;
          gui_result_storage = UI_NEXT_SONG;
          break;
        case UI_PREVIOUS_SONG: 
          /* if halfway through mod restart it, if
             beginning jump to the previous one */
          
          /*	  if (count_song < SMALL_DELAY && optind>1 )
            optind-=2;
          else 
            optind--;
          thisapp->change_song = TRUE;
          count_song=0;*/
        
          thisapp->change_song = TRUE;
          gui_result_storage = UI_PREVIOUS_SONG;
          gui_result = UI_NONE;
          break;
        case UI_QUIT:
          thisapp->doquit=1;
          break;
        case UI_JUMP_TO_NEXT_PATTERN:
#ifdef ULTRA
          if (use_card>=0) {
            gus_do_stop();
            gus_queue_abort_to_stop();
            gus_queue_flush();
          }       
#endif /* ULTRA */
          MP_NextPosition();
          gui_result = UI_NONE;
          break;
        case UI_JUMP_TO_PREV_PATTERN:
#ifdef ULTRA
          if (use_card>=0) {
            gus_do_stop();
            gus_queue_abort_to_stop();
            gus_queue_flush();
          }       
#endif /* ULTRA */
          if (count_pattern < SMALL_DELAY) 
            /* near start of pattern? */
            MP_PrevPosition();
          else
            MP_RestartPosition();
          count_pattern=0;
          gui_result = UI_NONE;
          break;

        case UI_SPEED_UP:
          if ((old_bpm*(speed_constant+0.05))<=255)
            speed_constant+=0.05;
                gui_result = UI_NONE;
          break;
        case UI_SLOW_DOWN:
          if ((old_bpm*(speed_constant-0.05))>10)
            speed_constant-=0.05;
                gui_result = UI_NONE;
          break;
        case UI_NORMAL_SPEED:
          speed_constant=1.0;
          break;
        case UI_VOL_UP:
          if(mp_volume<250)
            mp_volume+=5;
          break;
        case UI_VOL_DOWN:
          if(mp_volume>5)
            mp_volume-=5;
          break;
          
        case UI_NORMAL_VOL:
          mp_volume=100;
          break;
          
        default:
          break;
        }
        
        if ((old_bpm*speed_constant)>255)
          mp_bpm=255;
              else
          mp_bpm=(int) rint(old_bpm*speed_constant);

    }
    
  }
  
  /* Update effects */
  
  for(t=0;t<pf->numchn;t++){
    mp_channel=t;
    a=&mp_audio[t];
    PlayEffects();
  }
  
  
  for(t=0;t<pf->numchn;t++){
    INSTRUMENT *i;
    SAMPLE *s;
    SWORD envpan,envvol;
    
    a=&mp_audio[t];
    i=a->i;
    s=a->s;
    
    if(i==NULL || s==NULL) continue;
    
    if(a->period<40) a->period=40;
    if(a->period>8000) a->period=8000;
    
    if(a->kick){
      MD_VoicePlay(t,a->handle,a->start,s->length,s->loopstart,s->loopend,s->flags);
      a->kick=0;
      a->keyon=1;
      
      a->fadevol=32768;
      
      StartEnvelope(&a->venv,i->volflg,i->volpts,i->volsus,i->volbeg,i->volend,i->volenv);
      StartEnvelope(&a->penv,i->panflg,i->panpts,i->pansus,i->panbeg,i->panend,i->panenv);
    }
    
    envvol=ProcessEnvelope(&a->venv,256,a->keyon);
    envpan=ProcessEnvelope(&a->penv,128,a->keyon);
    
    tmpvol=a->fadevol;              /* max 32768 */
    tmpvol*=envvol;                 /* * max 256 */
    tmpvol*=a->volume;              /* * max 64 */
    tmpvol/=16384;                  /* tmpvol/(256*64) => tmpvol is max 32768 */
    
    tmpvol*=globalvolume;   /* * max 64 */
    tmpvol*=mp_volume;              /* * max 100 */
    tmpvol/=3276800UL;              /* tmpvol/(64*100*512) => tmpvol is max 64 */
    
    MD_VoiceSetVolume(t,tmpvol);
    
    if(s->flags& SF_OWNPAN){
      MD_VoiceSetPanning(t,DoPan(envpan,a->panning));
    }
    else{
      MD_VoiceSetPanning(t,a->panning);
    }
    
    if(pf->flags&UF_LINEAR)
      MD_VoiceSetFrequency(t,GetFreq2(a->period));
    else
      MD_VoiceSetFrequency(t,(3579546UL<<2)/a->period);
    
    /*  if key-off, start substracting
	fadeoutspeed from fadevol: */
    
    if(!a->keyon){
      if(a->fadevol>=i->volfade)
	a->fadevol-=i->volfade;
      else
	a->fadevol=0;
    }
  }
}	

void process(){
  
  thisapp->processEvents();
}

///////////////////////////////////////////////////////////////////////////
// DockIcon
///////////////////////////////////////////////////////////////////////////

DockWidget::DockWidget(const char *name)
  : QWidget(0, name, 0) {

  docked = false;

  pos_x = pos_y = 0;
  have_KModPlayer_position = false;
  QString pixdir = thisapp->kde_datadir() + "/kmodplayer/pics/";
  QString tmp;

#define PMERROR(pm) \
  tmp.sprintf(klocale->translate("Could not load %s !"), pm); \
  QMessageBox::warning(this, klocale->translate("Error"), tmp);


  if (!kmodplayer_pixmap.load(pixdir + "kmodsmall.xpm")){
    PMERROR("kmodsmall.xpm");
  }

  // popup menu for right mouse button
  popup_m = new QPopupMenu();

  popup_m->insertItem(klocale->translate("Play/Pause"), this, SLOT(play_pause()));
  popup_m->insertItem(klocale->translate("Stop"), this, SLOT(stop()));
  popup_m->insertItem(klocale->translate("Forward"), this, SLOT(forward()));
  popup_m->insertItem(klocale->translate("Backward"), this, SLOT(backward()));
  popup_m->insertItem(klocale->translate("Next"), this, SLOT(next()));
  popup_m->insertItem(klocale->translate("Previous"), this, SLOT(prev()));
  popup_m->insertItem(klocale->translate("Module List"), 
		      this, SLOT(eject()));
  popup_m->insertSeparator();
  popup_m->insertItem(klocale->translate("Quit"), this, SLOT(quit()));
  popup_m->insertSeparator();
  toggleID = popup_m->insertItem(klocale->translate("Restore"),
				 this, SLOT(toggle_window_state()));

  KDNDDropZone *dropZone = new KDNDDropZone( this, DndURL );
  connect( dropZone, SIGNAL( dropAction( KDNDDropZone* )), PlaylistDialog::playlistdialog, 
	   SLOT( onDrop( KDNDDropZone*)));

  QToolTip::add( this, "KModPlayer" );

}

DockWidget::~DockWidget() {
}

void DockWidget::dock() {

  if (!docked) {


    // prepare panel to accept this widget
    KWM::setDockWindow (this->winId());

    // that's all the space there is
    this->setFixedSize(24, 24);

    // finally dock the widget
    this->show();
    docked = true;
  }
  if(thisapp->mainWindow){
    QPoint point = thisapp->mainWindow->mapToGlobal (QPoint (0,0));
    pos_x = point.x();
    pos_y = point.y();

  }
}

void DockWidget::undock() {

  if (docked) {

    // the widget's window has to be destroyed in order 
    // to undock from the panel. Simply using hide() is
    // not enough.
    this->destroy(true, true);

    // recreate window for further dockings
    this->create(0, true, false);

    docked = false;
  }
}

const bool DockWidget::isDocked() {

  return docked;

}

void DockWidget::paintEvent (QPaintEvent *e) {

  (void) e;

  paintIcon();

}

void DockWidget::paintIcon () {

  bitBlt(this, 0, 0, &kmodplayer_pixmap);


}

void DockWidget::timeclick() {

  if(this->isVisible()){
    paintIcon();
  }  
}


void DockWidget::mousePressEvent(QMouseEvent *e) {

  // open/close connect-window on right mouse button 
  if ( e->button() == LeftButton ) {
    toggle_window_state();
  }

  // open popup menu on left mouse button
  if ( e->button() == RightButton  || e->button() == MidButton) {
    int x = e->x() + this->x();
    int y = e->y() + this->y();

    QString text;
    if(thisapp->mainWindow->isVisible())
      text = klocale->translate("Minimize");
    else
      text = klocale->translate("Restore");
    
    popup_m->changeItem(text, toggleID);
    popup_m->popup(QPoint(x, y));
    popup_m->exec();
  }

}

void DockWidget::toggle_window_state() {

  // restore/hide connect-window
  if(thisapp->mainWindow != 0L)  {
    if (thisapp->mainWindow->isVisible()){

     QPoint point = thisapp->mainWindow->mapToGlobal (QPoint (0,0));
     pos_x = point.x();
     pos_y = point.y();
     thisapp->mainWindow->hide();
     PlaylistDialog::playlistdialog->hide();
    }
    else {

      if(!have_KModPlayer_position)
	SaveKModPlayerPosition();

      thisapp->mainWindow->setGeometry(
		     pos_x, 
		     pos_y,
		     thisapp->mainWindow->width(),
		     thisapp->mainWindow->height());
      
      thisapp->mainWindow->show();
    }
  }
}

void DockWidget::SaveKModPlayerPosition(){

     QPoint point = thisapp->mainWindow->mapToGlobal (QPoint (0,0));
     pos_x = point.x();
     pos_y = point.y();
     have_KModPlayer_position = true;

}
void DockWidget::eject() {

  thisapp->mainWindow->ejectClicked();

}

void DockWidget::quit()
{
  thisapp->mainWindow->quitClicked();
}

void DockWidget::next() {

  thisapp->mainWindow->nextClicked();

}


void DockWidget::stop() {

  thisapp->mainWindow->stopClicked();

}

void DockWidget::prev() {

  thisapp->mainWindow->prevClicked();

}

void DockWidget::play_pause() {

  thisapp->mainWindow->playClicked();
}


void DockWidget::forward() {

  thisapp->mainWindow->fwdClicked();

}


void DockWidget::backward() {
  thisapp->mainWindow->bwdClicked();
}


// -- EOF --
