/*   
   kModPlayer - Qt-Based Mod Player
   
   $Id: kmodplayer.h,v 1.2 1997/10/25 23:23:32 wuebben Exp $
 
   portions: Copyright 1996 Bernd Johannes Wuebben math.cornell.edu
  
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

 
 */

#ifndef KMODPLAYER_H
#define KMODPLAYER_H

#include "version.h"

#include "bwlednum.h"

#include <qdstream.h> 
#include <qfile.h> 
#include <qtabdlg.h> 
#include <qfiledlg.h>
#include <qpushbt.h>
#include <qlabel.h>
#include <qdialog.h>
#include <qapp.h>
#include <qpopmenu.h>
#include <qtimer.h>
#include <qbitmap.h>
#include <qslider.h>
#include <qgrpbox.h>
#include <qcombo.h>
#include <qmsgbox.h>
#include <qscrbar.h>

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <pwd.h>
#include <sys/time.h>
#include <signal.h>
#include <sys/utsname.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/ioctl.h>

// #include "configdlg.h"

#include "ui.h"
#include <limits.h>
#include <sys/wait.h>

#include <kapp.h>
#include <ktmainwindow.h>
#include <kmsgbox.h>

#include "playlist.h"

#define PLAYLIST_WIDTH  550
#define PLAYLIST_HEIGHT 440

///////////////////////////////////////////////////////////////////////////

class KMod : public KTMainWindow {

	Q_OBJECT
public:

	~KMod();
	
	// Buttons

	QPushButton	*playPB;
	QPushButton	*stopPB;
	QPushButton	*prevPB;
	QPushButton	*nextPB;
	QPushButton	*fwdPB;
	QPushButton	*bwdPB;
	QPushButton	*quitPB;
	QPushButton	*replayPB;
	QPushButton	*ejectPB;
	QPushButton	*aboutPB;
	QPushButton *fasterPB;
	QPushButton *slowerPB;
	QPushButton *lowerBar;

	BW_LED_Number	*trackTimeLED[9];
    
	QPixmap folder_pixmap;
	QPixmap file_pixmap;
	QPixmap cdup_pixmap;

	QLabel	*statusLA;
	QLabel	*volLA;
	QLabel  *titleLA;
	QLabel  *propertiesLA;
	QLabel  *modlabel;
	QLabel  *looplabel;
	QLabel  *speedLA;
	QLabel  *song_count_label;


	QComboBox	*songListCB;
//	QScrollBar	*volSB; 

	QSlider		*volSB;
	QWidget 	*backdrop;
	QPushButton     *makeButton( int, int, int, int, const char * );

	int		mixerFd;
	int 		init_volume;

	QString		fileName;
	class Songlist *playlist;

	bool 		tooltips;
	// ConfigDlg       *configdlg;
	KConfig		*config;

	QString currmodname;
	QString currmodstring;

	// StoreVars 
	QString savevoltxt;
	QString savestatus;
	QString savetime;
	QString savelooplabel;
	QString saveprop;
	QString savecurrmodname;
	QString savecurrmodstring;
	QString savesongcount;
	QString savespeedtext;

protected:
	void closeEvent( QCloseEvent *e );  

private:
	void 		makedirs();
	QString 	getHomeDir();
	void 		loadPlaylist();
	void 		resetPos();
	void 		setLEDs(QString );
	void		drawPanel();
	void		cleanUp();
	void		loadBitmaps();
//	void		setLEDColor( QLEDNumber *, const QColor&,    const QColor&  );
	void		initMixer( const char *mixer = "/dev/mixer" );
	void            KillChild();
	void 		playtime();
	bool playthemod(QString);

  void updateLabel(QLabel *label,QString &oldtext,QString &newtext);
  void updateSongName();

public:
	void updateUI();
	KMod( QWidget *parent = 0, const char *name = 0 );

//	void 		GetVolume();
//	void            initSound( const char *mixer = "/dev/mixer" );
static	void 		MP_HandleTick(void);

public slots:
	void 		setColors();
  void 		PlayMOD();
	void		playClicked();
	void		stopClicked();
	void 		slowdownslot();
	void 		speedupslot();
	void		prevClicked();
	void		nextClicked();
	void		fwdClicked();
	void		bwdClicked();
	void		quitClicked();
	void		replayClicked();
	void		ejectClicked();
	void		aboutClicked();
	void		volChanged( int );
};

///////////////////////////////////////////////////////////////////////////

class DockWidget;
class KModPlayer : public KApplication {
  Q_OBJECT
public:
	KModPlayer(int argc, char **argv);
	void quit();
	~KModPlayer();

public:	
	enum ModPlayerState {
		Device_Error,
	  Ready,
		Stopped,
		Paused,
		Playing
	};
	
	ModPlayerState getState();
	void setState(ModPlayerState newstate);
  
public slots:
  bool startPlaying();
  void stopPlaying();

public:
  bool allocDevice();
	void freeDevice();

  void forcePlaying(int songnr);
	
private:
	ModPlayerState state;
	void initModplayer();
	void parseOptions(int nargc,char *nargv[]);

	bool deviceAlloced;
  void makedirs();
  QString getHomeDir();
	void PlayCommandlineMods();
	
public:
	bool loopmod;
	bool blink;
  bool doquit;
  bool change_song;
  bool go_next_mod;
  int vol;
  int	song_number ;

public:
	KMod *mainWindow;
  DockWidget *dockWindow;
	class UNIMOD 		*mod_ptr;
	// QTimer		*timer;
};

///////////////////////////////////////////////////////////////////////////

class DockWidget : public QWidget {
  Q_OBJECT

public:
  DockWidget(const char *name=0);
  ~DockWidget();

protected:
  void paintEvent(QPaintEvent *e);

private slots:
  void timeclick();
  void toggle_window_state();
  void play_pause();
  void forward();
  void backward();
  void next();
  void prev();
  void eject();
  void stop();
  void mousePressEvent(QMouseEvent *e);
  void quit();

public slots:
  void dock();
  void undock();
  void paintIcon();

public:
  const bool isDocked();
  void SaveKModPlayerPosition();

private:
  bool docked;
  int toggleID;
  int pos_x;
  int pos_y;
  QPopupMenu *popup_m;
  bool have_KModPlayer_position;
  QPixmap kmodplayer_pixmap;

};

#endif KMODPLAYER_H
