/*

Name:
MIKMOD.C

Description:
Modplaying example of mikmod - bare linux (unix?) version.

*/
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include "mikmod.h"
#include <curses.h>
#include "ui.h"

char *d_text[]={
	"           ",	/*standard, not deleted*/
	"<del>      ",	/*marked for deletion*/
	"<deleted>  ",	/*deleted, gone!*/
	"<del error>"	/*can't delete...*/
	};

char helptext[]=

"Available switches (CaSe SeNsItIvE!):\n"
"\n"
"  -d x    use device-driver #x for output (0 is autodetect). Default=0\n"
"  -ld     List all available device-drivers\n"
"  -ll     List all available loaders\n"
"  -x      disables protracker extended speed\n"
"  -p      disables panning effects (9fingers.mod)\n"
"  -v xx   Sets volume from 0 (silence) to 100. Default=100\n"
"  -f xxxx Sets mixing frequency. Default=44100\n"
"  -m      Force mono output (so sb-pro can mix at 44100)\n"
"  -8      Force 8 bit output\n"
"  -i      Use interpolated mixing\n"
"  -r      Restart a module when it's done playing";

#ifdef ULTRA

extern int use_card;

void processPosChange(struct echoData *echo) 
{
	extern char output_buf_4[256];
	sprintf(output_buf_4, "\rPattern/Pos: %03i/%03i ", echo->data1, echo->data2);
	printw("%s",output_buf_4);
	refresh();
}

void processFreqChange(struct echoData *echo) 
{
}

#endif /*ULTRA*/

void tickhandler(void)
{
	MP_HandleTick();    /* play 1 tick of the module */
	MD_SetBPM(mp_bpm);
}

#if (defined(NEEDS_USLEEP))
/*
 *  NAME:
 *      usleep     -- This is the precision timer for Test Set
 *                    Automation. It uses the select(2) system
 *                    call to delay for the desired number of
 *                    micro-seconds. This call returns ZERO
 *                    (which is usually ignored) on successful
 *                    completion, -1 otherwise.
 *
 *  ALGORITHM:
 *      1) We range check the passed in microseconds and log a
 *         warning message if appropriate. We then return without
 *         delay, flagging an error.
 *      2) Load the Seconds and micro-seconds portion of the
 *         interval timer structure.
 *      3) Call select(2) with no file descriptors set, just the
 *         timer, this results in either delaying the proper
 *         ammount of time or being interupted early by a signal.
 *
 *  HISTORY:
 *      Added when the need for a subsecond timer was evident.
 *
 *  AUTHOR:
 *      Michael J. Dyer                   Telephone:   AT&T 414.647.4044
 *      General Electric Medical Systems        GE DialComm  8 *767.4044
 *      P.O. Box 414  Mail Stop 12-27         Sect'y   AT&T 414.647.4584
 *      Milwaukee, Wisconsin  USA 53201                      8 *767.4584
 *      internet:  mike@sherlock.med.ge.com     GEMS WIZARD e-mail: DYER
 */

#include <unistd.h>
#include <errno.h>
#include <time.h>
#include <sys/time.h>
#include <sys/param.h>
#include <sys/types.h>

int     usleep( unsigned long int microSeconds )
{
        unsigned int            Seconds, uSec;
        int                     nfds, readfds, writefds, exceptfds;
        struct  timeval         Timer;
        nfds = readfds = writefds = exceptfds = 0;

        if( (microSeconds == (unsigned long) 0)
                || microSeconds > (unsigned long) 4000000 )
        {
                errno = ERANGE;         /* value out of range */
                perror( "usleep time out of range ( 0 -> 4000000 ) " );
                return -1;
        }

        Seconds = microSeconds / (unsigned long) 1000000;
        uSec    = microSeconds % (unsigned long) 1000000;

        Timer.tv_sec            = Seconds;
        Timer.tv_usec           = uSec;

        if( select( nfds, &readfds, &writefds, &exceptfds, &Timer ) < 0 )
        {
                perror( "usleep (select) failed" );
                return -1;
        }

        return 0;
}
#endif /*NEEDS_USLEEP*/

int main(int nargc,char *nargv[])
{
	extern int quit;
	extern int current;
	UNIMOD *mf;
	int cmderr=0;                   /* error in commandline flag */
	int morehelp=0;                 /* set if user wants more help */
	int t;
	extern char current_file[1024];
	extern char output_buf_1[256]; /* output_buf_1-4 used for storing current output */
	extern char output_buf_2[256]; /* ready to be reprinted after pause */
	extern char output_buf_3[1024];
	extern char output_buf_4[256];
	extern int deleted;
	extern float speed_constant;   /* multiplier for tempo,initialised to 1 */
	UWORD reppos;
	puts(mikbanner);
	/*
		Initialize soundcard parameters.. you _have_ to do this
		before calling MD_Init(), and it's illegal to change them
		after you've called MD_Init()
	*/

	current=1;		/* we are playing _this_ mod; jump to next 
				   when this goes to 0 */
	md_mixfreq      =44100;                     /* standard mixing freq */
	md_dmabufsize   =32768;                     /* standard dma buf size */
	md_mode         =DMODE_16BITS|DMODE_STEREO; /* standard mixing mode */
	md_device       =0;                                                     /* standard device: autodetect */

	/*
		Register the loaders we want to use..
	*/

	ML_RegisterLoader(&load_m15);    /* if you use m15load, register it as first! */
	ML_RegisterLoader(&load_mod);
	ML_RegisterLoader(&load_mtm);
	ML_RegisterLoader(&load_s3m);
	ML_RegisterLoader(&load_stm);
	ML_RegisterLoader(&load_ult);
	ML_RegisterLoader(&load_uni);
	ML_RegisterLoader(&load_xm);

	/*
		Register the drivers we want to use:
	*/

        MD_RegisterDriver(&drv_nos);
	MD_RegisterDriver(&drv_raw);

#ifdef SUN
	MD_RegisterDriver(&drv_sun);
#elif defined(SOLARIS)
	MD_RegisterDriver(&drv_sun);
#elif defined(__alpha)
        MD_RegisterDriver(&drv_AF);
#elif defined(LINUX)
        MD_RegisterDriver(&drv_vox);
	#ifdef ULTRA
	       MD_RegisterDriver(&drv_ultra);
	#endif /* ULTRA */
#elif defined(__hpux)
        MD_RegisterDriver(&drv_hp);
#elif defined(AIX)
        MD_RegisterDriver(&drv_aix);
#elif defined(SGI)
        MD_RegisterDriver(&drv_sgi);
#endif

	MD_RegisterPlayer(tickhandler);

	/* Parse option switches using standard getopt function: */

	opterr=0;

	while( !cmderr &&
		  (t=getopt(nargc,nargv,"ohxpm8irv:f:l:d:")) != EOF ){

		switch(t){

			case 'd':
				md_device=atoi(optarg);
				break;

			case 'l':
				if(optarg[0]=='d') MD_InfoDriver();
				else if(optarg[0]=='l') ML_InfoLoader();
				else{
					cmderr=1;
					break;
				}
				exit(0);

			case 'r':
				mp_loop=1;
				break;

			case 'm':
				md_mode&=~DMODE_STEREO;
				break;

			case '8':
				md_mode&=~DMODE_16BITS;
				break;

			case 'i':
				md_mode|=DMODE_INTERP;
				break;

			case 'x':
				mp_extspd=0;
				break;

			case 'p':
				mp_panning=0;
				break;

			case 'v':
				if((mp_volume=atoi(optarg))>100) mp_volume=100;
				break;

			case 'f':
				md_mixfreq=atol(optarg);
				break;

			case 'h':
				morehelp=1;
				cmderr=1;
				break;

			case '?':
				puts("\07Invalid switch or option needs an argument\n");
				cmderr=1;
				break;
		}
	}

	if(cmderr || optind>=nargc){

		/*
			there was an error in the commandline, or there were no true
			arguments, so display a usage message
		*/

                printf("Usage: %s [switches] <fletch.mod> ... \n",nargv[0]);

                if(morehelp)
                        puts(helptext);
                else
                        printf("Type %s -h for more help.\n",nargv[0]);

                exit(-1);
	}


	/*  initialize soundcard */

	if(!MD_Init()){
		printf("Driver error: %s.\n",myerr);
		return 0;
	}

	/*  initialize volume and tempo multipliers */

	speed_constant=1.0;
	mp_volume=100;


#ifdef __hpux
		/* without the following line, quiting mikmod by CTRL-C
		 * would close the terminal window, which is probably
		 * not intended by the user
		 */
		signal(SIGINT, SIG_IGN);
#endif

	/*  initialize curses interface */

	initscr(); 
	cbreak(); 
	noecho(); 
	nonl(); 
	nodelay(stdscr, TRUE);
	keypad(stdscr, TRUE); 

	for(quit=0; !quit && optind<nargc; optind++){

		strcpy(current_file,nargv[optind]);
		sprintf(output_buf_1,"%s: %d bit %s %s, %u Hz\n",
			md_driver->Name,
			(md_mode&DMODE_16BITS) ? 16:8,
			(md_mode&DMODE_INTERP) ? "interpolated":"normal",
			(md_mode&DMODE_STEREO) ? "stereo":"mono",
			md_mixfreq);
		printw("%s",output_buf_1);

		/* load the module */

		mf=ML_LoadFN(nargv[optind]);

		/* didn't work -> exit with errormsg. */

		if(mf==NULL){
			printf("MikMod Error: %s\n",myerr);
			break;
		}

		/*      initialize modplayer to play this module */

		MP_Init(mf);

		sprintf(output_buf_2,"                                                                                                                                                     ");

		sprintf(output_buf_2,"File: %s%s\n",current_file,d_text[deleted]); 

		printw("%s",output_buf_2);

		sprintf(output_buf_3,"Name: %s\n"
				"Type: %s, "
				"Periods: %s,%s\n",
				mf->songname,
				mf->modtype,
				(mf->flags&UF_XMPERIODS) ? "XM type" : "mod type",
				(mf->flags&UF_LINEAR) ? "Linear" : "Log");
		printw("%s",output_buf_3);

		
		/*	set the number of voices to use.. you
			could add extra channels here (e.g. md_numchn=mf->numchn+4; )
			to use for your own soundeffects:
		*/

		md_numchn=mf->numchn;

		/*  start playing the module: */

		MD_PlayStart();

		while(!MP_Ready() && !quit){ /* if we have a quit signal, exit loop */

			MD_Update();

			/* no need to wait with the unix drivers */

#ifndef ULTRA
			sprintf(output_buf_4,"\rpat:%2d/%d pos:%2d/64 spd:%d bpm:%3.0f%% vol:%d%% ",mp_sngpos+1,mf->numpos,mp_patpos,mp_sngspd,speed_constant*100,mp_volume);
			printw("%s",output_buf_4);
#endif /* ULTRA */
			if(!current){ 		/*current=0 when the next or previous*/
				current=1;	/*mod is selected*/
				break;
			}

			refresh(); /* without this, the display is not necessarily updated! */
		}
		MD_PlayStop();          /* stop playing */
		ML_Free(mf);            /* and free the module */

		printw("\n");
		clear();
	}

	MD_Exit();
	endwin();
	if (quit)
		{
		return 0;
		}
	printf("Finished playlist...\n");
	return 1;
}
