/*
   kModPlayer - Qt-Based Mod Player
   
   $Id: playlist.cpp,v 1.4 1997/10/25 23:23:32 wuebben Exp $

   Copyright 1996 Bernd Johannes Wuebben math.cornell.edu
             1998 Emmeran "Emmy" Seehuber <es@mylius.com>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include "playlist.h"
#include <stdio.h>

#include "kmodplayer.h"
#include "kfiledialog.h"
#include "kltools.h"

#include <drag.h>
#include <kurl.h>

#define BORDER_WIDTH 2

extern KModPlayer *thisapp;
extern QString current_playlist;
extern int playlist_x;
extern int playlist_y;

KSaveChdir::KSaveChdir(const char *newdir)
{
  getcwd(buff,buffsize);
  chdir(newdir);
}

KSaveChdir::~KSaveChdir()
{
  chdir(buff);
}

void Songlist::loadSonglist( const char *filename ) 
{
  KSaveChdir mychdir(savedir);
  QFileInfo info(filename);
  info.setFile(filename);
  if(!info.exists() || !info.isReadable())
    return;
  
  QFile f(filename);

  f.open( IO_ReadWrite | IO_Translate);

  char buffer[1024];
  QString tempstring;

  while(!f.atEnd()) {
    buffer[0] = (char) 0;
    f.readLine(buffer,511);

    tempstring = buffer;
    tempstring = tempstring.stripWhiteSpace();
    if (!tempstring.isEmpty()){
      append(tempstring.copy());
    }
  }

  f.close();
}

void Songlist::saveSonglist(const char *filename)
{
  QFile f(filename);
  KSaveChdir mychdir(savedir);

  if( f.open( IO_ReadWrite | IO_Translate | IO_Truncate ) ) {
    QString tempstring;

    for (int i = 0; i < (int)count(); i++){
      tempstring = at(i);
      tempstring += '\n';
      f.writeBlock(tempstring.data(),tempstring.length());
    }

    f.close();
  }
}

PlaylistDialog::PlaylistDialog(QWidget *, const char *,Songlist *thesonglist)
  : KLWindow() 
{

  // Init DnD: Set up drop zone and drop handler
  dropZone = new KDNDDropZone( this, DndURL );
  connect( dropZone, SIGNAL( dropAction( KDNDDropZone* )), 
	   SLOT( onDrop( KDNDDropZone*)));

  connect( this, SIGNAL(iconify()), SLOT(onIconify()));

  ASSERT( playlistdialog == 0 );
  playlistdialog = this;

  setCaption("Module List");

  QPopupMenu *file = new QPopupMenu;
  CHECK_PTR( file );

  file->insertItem( "New ",  this, SLOT(newPlaylist()));
  file->insertItem( "Open ",  this, SLOT(loadPlaylistFD()));
  file->insertItem( "Save ", this, SLOT(savePlaylist()));
  file->insertItem( "Save as",  this, SLOT(saveasPlaylist()));
  file->insertSeparator();
  file->insertItem( "Close", this, SLOT(onIconify()));
  
  QPopupMenu *edit = new QPopupMenu;
  CHECK_PTR( edit );
  edit->insertItem( "Add", this, SLOT(addEntry()) );
  edit->insertItem( "Remove", this, SLOT(removeEntry()) );
  
  QPopupMenu *help = new QPopupMenu;
  CHECK_PTR( help );
  help->insertItem( "Help", this, SLOT(help()));

  KMenuBar *menu = menuBar();
  CHECK_PTR( menu );
  menu->insertItem( "&File", file );
  menu->insertItem( "&Edit", edit );
  menu->insertSeparator();
  menu->insertItem( "&Help", help );

  KLButton *addbutton, *rembutton, *newbutton, 
           *loadbutton, *savebutton, *saveasbutton,
           *closebutton;
  KLGroup &group = (*(new KLHVGroup(false))
                   << (*(new KLHVGroup(true)) 
                     << (*(listbox = new KLListBox()) << setYWeight << 300 << setXWeight << 200 ) // Just let it get a little bigger ...
                     << (*(new KLHVGroup(false))
                        << (*(new KLGroupBox("Mode"))
                           << new KLRadioButton("Play",&rm,0)
                           << new KLRadioButton("Insert",&rm,1)
                           << new KLRadioButton("Remove",&rm,2))
                        << (filenameonly = new KLCheckBox("Filename only"))
                        << (randomcheck = new KLCheckBox("Random Play"))
                        << (*(new KLHVSpace()) << setFixXSize << 20)
                        << (addbutton = new KLButton("Add"))
                        << (rembutton = new KLButton("Remove"))
                        << (*(new KLHVSpace()) << setFixXSize << 20)
                        << (newbutton = new KLButton("New"))
                        << (loadbutton = new KLButton("Load"))
                        << (savebutton = new KLButton("Save"))
                        << (saveasbutton = new KLButton("Save as"))
                        << new KLHVSpace()
                        << (closebutton = new KLButton("Close window"))
                        << setWeight << 1))
                  << (playlistNameLabel = new KLLabel()));
  setRootChild(&group);
  rm.setChecked(0l);

  connect(listbox, SIGNAL(selected(int)), this, 
	    SLOT(selectAction()));

  connect(addbutton,SIGNAL(clicked()),this,SLOT(addEntry()));
  connect(rembutton,SIGNAL(clicked()),this,SLOT(removeEntry()));
  connect(newbutton,SIGNAL(clicked()),this,SLOT(newPlaylist()));
  connect(loadbutton,SIGNAL(clicked()),this,SLOT(loadPlaylistFD()));
  connect(savebutton,SIGNAL(clicked()),this,SLOT(savePlaylist()));
  connect(saveasbutton,SIGNAL(clicked()),this,SLOT(saveasPlaylist()));
  connect(closebutton,SIGNAL(clicked()),this,SLOT(onIconify()));
  connect(filenameonly,SIGNAL(clicked()),this,SLOT(refillList()));

	// statusbar = statusBar();
	// statusbar->insertItem("",0);

  songlist = thesonglist;
  applyList();
  
  QString home = QDir::homeDirPath();
  home = home + "/.kde/share/apps/kmodplayer";
  QDir savedir(home.data());
  if(!savedir.exists()){
    savedir.mkdir(home.data());
  }
  songlist->savedir = home + "/";

  playlistNameLabel->setWeight(100);
  playlistNameLabel->setFrameStyle(QFrame::Panel | QFrame::Sunken);
  playlistNameLabel->setText("Current Playlist: " + current_playlist);
  // statusbar->changeItem(,0);
}

PlaylistDialog *PlaylistDialog::playlistdialog = 0;

PlaylistDialog::~PlaylistDialog()
{
    ASSERT( playlistdialog == this );  
    playlistdialog = 0;
}

void PlaylistDialog::selectSong(int songnum)
{
  listbox->setCurrentItem(songnum);
}


void PlaylistDialog::loadPlaylistFD()
{
  KSaveChdir(songlist->savedir);
  QString filename = KFileDialog::getOpenFileName(songlist->savedir,0,0,current_playlist);
  if( filename.isEmpty() )
    return;
  current_playlist = filename;
  loadPlaylist(current_playlist);
}


void PlaylistDialog::createDialog(Songlist *songlist)
{
    new PlaylistDialog(0,0,songlist); // Automaticly sets the playlistdialog-Variable
}


void PlaylistDialog::showPlaylist()
{
  if( playlistdialog ) {
    playlistdialog->show();
    playlistdialog->raise();
  }
  else  {
    ASSERT( FALSE );
  }
}

void PlaylistDialog::closeEvent( QCloseEvent *e )
{
  e->ignore();                            
  hide(); // just hide
}

void PlaylistDialog::selectAction()
{
  switch( rm.checked() ) 
  {
    case 0: {
      if( (thisapp->song_number == listbox->currentItem()) && (thisapp->getState() == KModPlayer::Playing) )
        return;
      thisapp->song_number = listbox->currentItem();
      thisapp->go_next_mod = FALSE;
      thisapp->change_song = TRUE;
      if( (thisapp->getState() == KModPlayer::Playing) || (thisapp->getState() == KModPlayer::Paused) ) 
        thisapp->setState( KModPlayer::Playing );
      else
        thisapp->mainWindow->playClicked();
    
    }; break;
    case 1: {
      QString filename = KFileDialog::getOpenFileName();
      if( filename.isEmpty() )
        return;
      
      songlist->insert(listbox->currentItem(), filename);
      listbox->insertItem(convertToListName(filename),listbox->currentItem());
    }; break;
    case 2: {
      removeEntry();
    }; break;
  }
}

QString PlaylistDialog::convertToListName(QString filename)
{
  if( filenameonly->isChecked() ) {
    QFileInfo fi(filename);
    return fi.fileName();
  }
  return filename;
}


void PlaylistDialog::newPlaylist()
{ 
  listbox->clear();
  songlist->clear();
  applyList();
}


void PlaylistDialog::addEntry()
{
  QString filename = KFileDialog::getOpenFileName();
  if( filename.isEmpty() )
    return;
  songlist->append(filename);
  addListBoxItem(filename);
}


void PlaylistDialog::onDrop( KDNDDropZone* _zone )
{
  QStrList strlist;

  unsigned int songnr = songlist->count();
  strlist = _zone->getURLList();
  for(unsigned int i=0; i < strlist.count(); i++ ) {
    KURL *url = new KURL( strlist.at(i) );
    QString urlQstr  = url->path();
    QString &urlQref = urlQstr;
    KURL::decodeURL(urlQref);
    songlist->append(urlQstr.data());
    addListBoxItem(urlQstr.data());
    delete url;
  }
  if( songnr < songlist->count() ) 
    thisapp->forcePlaying(songnr);
}


void PlaylistDialog::removeEntry()
{

  if (listbox->currentItem() != -1){
    songlist->remove(listbox->currentItem());
    listbox->removeItem(listbox->currentItem());
  }
}


void PlaylistDialog::addListBoxItem( const char *modpath )
{
    listbox->insertItem( convertToListName(modpath), -1);
}

void PlaylistDialog::applyList()
{
  listbox->clear();
  if (songlist->count()){
    for(int index = 0; index < (int)songlist->count(); index ++)
      addListBoxItem(songlist->at(index));
  }
}


void PlaylistDialog::savePlaylist()
{
  savePlaylistbyName(current_playlist);
}

void PlaylistDialog::savePlaylistbyName(QString name)
{
  if(name.isEmpty())
    return;

  songlist->saveSonglist(name);
}

void PlaylistDialog::saveasPlaylist()
{
  KSaveChdir(songlist->savedir);
  QString filename = KFileDialog::getSaveFileName(songlist->savedir,0,0,current_playlist);
  if( filename.isEmpty() )
    return;
  current_playlist = filename;
  savePlaylistbyName(current_playlist);
}


void PlaylistDialog::loadPlaylist(QString name)
{
  current_playlist = name;
  songlist->clear();
  songlist->loadSonglist(name);
  applyList();
 
  // statusbar->changeItem("Current Playlist: " + current_playlist,0);
  playlistNameLabel->setText("Current Playlist: " + current_playlist);
}

void PlaylistDialog::onIconify()
{
  hide();
}

void PlaylistDialog::refillList()
{
  applyList();
}

void PlaylistDialog::help(){

  if(!thisapp)
    return;

  thisapp->invokeHTMLHelp("kmodplayer/kmodplayer.html","");
}

