/*
   kModPlayer - Qt-Based Mod Player
   
   $Id: playlist.h,v 1.2 1997/10/25 23:23:32 wuebben Exp $

   Copyright 1996 Bernd Johannes Wuebben math.cornell.edu

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


*/

#ifndef PLAY_LIST_H
#define PLAY_LIST_H

#include <qstrlist.h> 
#include "klwindow.h"
#include "kllistbox.h"
#include "klbutton.h"
#include "klgroup.h"
#include "klgroupbox.h"
#include "klradiobutton.h"
#include "klcheckbox.h"

class KSaveChdir {
public:
  KSaveChdir(const char *newdir);
  ~KSaveChdir();
private:
  const int buffsize = 4096;
  char buff[buffsize + 1];
};


class Songlist : public QStrList {
public:
	Songlist() : QStrList(true)  {};
	void loadSonglist(const char *filename); // Appends the file to the current songlist !
	void saveSonglist(const char *filename);
public:
  QString savedir;
};

class KDNDDropZone;

class PlaylistDialog : public KLWindow {
Q_OBJECT
public:
   PlaylistDialog(QWidget *parent, const char *name,Songlist *songlist);
   ~PlaylistDialog();
private slots:
  void saveasPlaylist();
  void help();
  void savePlaylist();
  void savePlaylistbyName(QString name);
  void removeEntry();
  void addEntry();
	void selectAction();
  void newPlaylist();
  void onDrop( KDNDDropZone* _zone );
  void closeEvent( QCloseEvent *e );
  void onIconify();
  void loadPlaylistFD();
  void refillList();

  void addListBoxItem( const char *modpath );

protected:
  void loadPlaylist(QString name);

private:
    class KDNDDropZone	*dropZone;
    // KMenuBar *menu;    
    KLListBox* listbox;
    KLCheckBox *randomcheck;
    KLLabel *playlistNameLabel;
    // KStatusBar *statusbar;
    Songlist*  songlist;
    KRadioManager rm;
    KLCheckBox *filenameonly;
    QString PlaylistDialog::convertToListName(QString filename);

public:
    bool isRandomPlay() { return randomcheck->isChecked(); };
    void selectSong(int songnum);
    
public:
    static void createDialog(Songlist *songlist);
    static void showPlaylist();
    static PlaylistDialog *playlistdialog;
    void applyList();
};


class MyButton : public QPushButton 
{
	public:
	MyButton(QWidget *parent, const char *id,const char *text,int w, int h) : QPushButton(parent,id)
	{
		setText(text);
		setMaximumSize(w,h);
	}
};


#endif /* PLAY_LIST_H */
